#
# AttachmentFile.py
#
# Clase para almacenar un archivo adjunto y sus cabeceras
# correspondientes.
#
# Alejandro Álvarez Ayllón - 28/10/2006
#
from ZODB.PersistentMapping import PersistentMapping
from Persistence import Persistent
from AttachmentFile import AttachmentFile
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseContent

from config import PRIORITY_NAME

import time
import locale

class Bug(Persistent):
    """
    Clase Bug.
    Corresponde con una notificación de un bug.
    """

    security = ClassSecurityInfo()
        
    # Inicialización
    def __init__(self, parent, id, author, title, description, priority, attachment = None):
        """
        Descripción: Inicializar y crear.
        Parámetros:  parent  = Objeto propietario de este.
                     author  = Quién ha enviado el bug.
                     title   = Título del bug.
                     description = Descripción detallada.
                     attachment  = Tipo FileUpload.
        Devuelve:    Nada.
        """

        self.parent       = parent
        self.id           = id
        self.submittedBy  = author
        self.assignedTo   = None
        self.status       = 'open'
        self.title        = title
        self.description  = description
        self.submitted    = time.localtime()
        if attachment.filename != '':
            self.attachment   = AttachmentFile(attachment)
        else:
            self.attachment = None
        
        if priority > 5:
            priority = 5
        elif priority < 0:
            priority = 0
            
        self.priority    = priority

    # Acceso a las variables
    security.declarePublic('getId')
    def getId(self):
        return self.id
    
    security.declarePublic('getSubmittedBy')
    def getSubmittedBy(self):
        return self.submittedBy

    security.declarePublic('getAssignedTo')
    def getAssignedTo(self):
        return self.assignedTo
    
    security.declarePublic('getStatus')
    def getStatus(self):
        return self.status
    
    security.declarePublic('getTitle')
    def getTitle(self):
        return self.title
    
    security.declarePublic('getDescription')
    def getDescription(self):
        return self.description
    
    security.declarePublic('getSubmitted')
    def getSubmitted(self):
        return time.strftime(locale.nl_langinfo(locale.D_T_FMT),
                             self.submitted)
    
    security.declarePublic('getAttachment')
    def getAttachment(self):
        return self.attachment

    security.declarePublic('getPriority')
    def getPriority(self):
        """
        Descripción: Devuelve la prioridad con su nombre (en inglés)
        Parámetros:  Ninguno.
        Devuelve:    Very low(0)....Critical(5)
        """

        return PRIORITY_NAME[self.priority]

    security.declarePublic('getPriorityValue')
    def getPriorityValue(self):
        return self.priority

    # Asignar el bug
    security.declarePublic('assign')
    def assign(self, user):
        """
        Descripción: Asigna el bug a un usuario. Sólo puede hacerlo el propietario
                     o un administrador.
        Parámetros:  user = El usuario al que se le asigna.
        Devuelve:    Nada.
        """

        if self.parent.isAdmin():
            self.assignedTo = user
            self.status     = 'assigned'
            return 'Assigned'
        else:
            return 'You don''t have enough permissions'

    # Darlo por cerrado
    security.declarePublic('close')
    def close(self):
        """
        Descripción: Cierra el bug. Sólo puede cerrarlo a quien se le ha asignado o
                     alguien con permisos de administración.
        Parámetros:  Ninguno.
        Devuelve:    Nada.
        """

        if self.status == 'closed':
            return 'The item is already closed'
        elif self.allowedToClose(): 
            self.status = 'closed'
            return 'closed'
        else:
            return 'You are not allowed to close the item'

    # Indica si tiene permisos para cerrar
    security.declarePublic('allowedToClose')
    def allowedToClose(self):
        """
        Descripción: Indica si el usuario actual tiene permiso para cerrar
                     el bug.
        Parámetros:  Ninguno.
        Devuelve:    Un booleano.
        """
        
        uid = getSecurityManager().getUser().getId()
        
        return uid and ((self.assignedTo and uid == self.assignedTo) or self.parent.isAdmin())
