#
# ATProjectFolder.py
#
# Carpeta contenedora de proyectos (ATProject)
#
# Alejandro Álvarez Ayllón - 13/04/2006
#

from Products.Archetypes.public import registerType
from Products.CMFCore import CMFCorePermissions

from AccessControl import ClassSecurityInfo

import Utils

from config import PROJECTNAME, I18N_DOMAIN
from config import OPERATIVE_SYSTEM_LIST, PROGRAMMING_LANGUAGE_LIST
from config import TOPIC_LIST, TRANSLATION_LIST, LICENSE_LIST
from config import DEVELOPMENT_STATUS_LIST

from unicodedata import normalize

# Funciones internas para comprobar los parámetros
def checkStringFilter(string, list):
    """
    Descripción: Indica si la cadena está en la lista.
    Parámetros:  string = La cadena que se quiere comprobar.
                 list   = La lista/tupla donde buscar.
    Devuelve:    True si está o la lista está vacía. False si no.
    """
    if len(list) == 0:
        return True

    return string.encode() in list

def checkListFilter(a, b):
    """
    Descripción: Indica si al menos uno de los elementos de 'a' está en 'b'.
    Parámetros:  a,b = Dos listas/tuplas/vectores.
    Devuelve:    True si algún elemento de a está en b, o si b está vacía.
                 False si no.
    """
    if len(b) == 0:
        return True

    for i in a:
        if i.encode() in b:
            return True

    return False
    

# Clase ATProjectFolder
class ATProjectFolder(Utils.Content, Utils.BaseBTreeFilteredFolder):
    """Project Folder"""

    content_icon    = 'projectfolder_icon.gif'
    meta_type       = 'ATProjectFolder'
    portal_type     = 'ProjectFolder'
    archetype_name  = 'Project Folder'
    typeDescription = 'Project folder'

    __implements__ = Utils.BaseFilteredFolder.__implements__

    actions = ({
        'id'         : 'view',
        'name'       : 'View',
        'action'     : 'string:${object_url}/projectfolder',
        'permissions': (CMFCorePermissions.View,)
        },
               )

    security = ClassSecurityInfo()

    allowedTypes = ['Project',]

    # Devuelve los valores posibles de los campos
    security.declareProtected(CMFCorePermissions.View, 'getDevelopmentStatusList')
    def getDevelopmentStatusList(self):
        """
        Descripción: Devuelve la lista de posibles estados de desarrollo.
        Parámetros:  Ninguno.
        Devuelve:    La lista de estados de desarrollo.
        """
        return DEVELOPMENT_STATUS_LIST

    security.declareProtected(CMFCorePermissions.View, 'getLicenseList')
    def getLicenseList(self):
        """
        Descripción: Devuelve la lista de posibles licencias.
        Parámetros:  Ninguno.
        Devuelve:    La lista de licencias.
        """
        return LICENSE_LIST

    security.declareProtected(CMFCorePermissions.View, 'getOperativeSystemList')
    def getOperativeSystemList(self):
        """
        Descripción: Devuelve la lista de posibles sistemas operativos.
        Parámetros:  Ninguno.
        Devuelve:    La lista de sistemas operativos.
        """
        return OPERATIVE_SYSTEM_LIST

    security.declareProtected(CMFCorePermissions.View, 'getProgrammingLanguageList')
    def getProgrammingLanguageList(self):
        """
        Descripción: Devuelve la lista de lenguajes de programación.
        Parámetros:  Ninguno.
        Devuelve:    La lista de lenguajes de programación.
        """
        return PROGRAMMING_LANGUAGE_LIST

    security.declareProtected(CMFCorePermissions.View, 'getTopicList')    
    def getTopicList(self):
        """
        Descripción: Devuelve la lista de temas.
        Parámetros:  Ninguno.
        Devuelve:    La lista de temas.
        """
        return TOPIC_LIST

    security.declareProtected(CMFCorePermissions.View, 'getTranslationList')
    def getTranslationList(self):
        """
        Descripción: Devuelve la lista de traducciones posibles.
        Parámetros:  Ninguno.
        Devuelve:    La lista de posibles idiomas.
        """
        return TRANSLATION_LIST

    # Devuelve los proyectos
    # Los campos para filtrar los resultados son opcionales
    security.declareProtected(CMFCorePermissions.View, 'getProjectList')    
    def getProjectList(self, dev_status = [], licenses = [], os = [],
                       language = [], topic = [], translation = [],
                       page = 0, pagesize = 10):
        """
        Descripción: Devuelve los proyectos que concuerdan con los términos.
        Parámetros:  Los distintos filtros en función de los campos de los proyectos.
                     Si se pasan listas vacías o no se pasan, se ignorarán.
        Devuelve:    Los proyectos contenidos que concuerdan con los filtros.
        """
        from math import floor

        result = []
        self.match_count  = 0
        self.page_size    = pagesize
        self.page         = page     = int(page)

        for i in self.objectValues(['ATProject',]):
            if checkStringFilter(i.status,          dev_status ) and \
            checkStringFilter(i.license,            licenses   ) and \
            checkListFilter(i.operative_system,     os         ) and \
            checkListFilter(i.programming_language, language   ) and \
            checkListFilter(i.topic,                topic      ) and \
            checkListFilter(i.translation,          translation):
                if floor(self.match_count / pagesize == page):
                    result.append(i)
                self.match_count = self.match_count + 1

        return result

    # Devuelve el paginador (en HTML)
    security.declareProtected(CMFCorePermissions.View, 'putPaginator')
    def putPaginator(self):
        """
        Descripción: Devuelve el código en HTML del paginador. Conserva los parámetros.
                     Es necesario llamarla después de llamar a getProjectList. 
        Parámetros:  Ninguno.
        Devuelve:    El código en HTML del paginador.
        """
        from math import ceil

        # Parámetros de la búsqueda
        params = ''
        if self.REQUEST is not None:
            for i in self.REQUEST.form.keys():
                if i != 'page':
                    params = '%s%s=%s&' % (params, i, self.REQUEST.form[i])

        # Generar las páginas
        code = ''
        for i in range(ceil(self.match_count / float(self.page_size))):
            url = '?%spage=%i' % (params, i)
            if i != self.page:
                code = '%s<a href="%s">%i</a>&nbsp;' % (code, url, i + 1)
            else:
                code = '%s<span class="selected">%i</span>&nbsp;' % (code, i + 1)
            
        return code

# Registrar el tipo
registerType(ATProjectFolder, PROJECTNAME)
